// -*- C++ -*-
//
// Package:     Arich
// Module:	
//
// Description: 
//
// Usage:
//      [class and public member function
//       documentation goes here]
//
// Author:      R. Pestotnik
// Created:     Thu May  1 00:00:01 EST 2003
// $Id$
//
// Revision history
//
// $Log$


#include <iostream>
#include "ArichDetector.hh"
#include "ArichAerogel.hh"

#include "ArichGeo.hh"

ArichAerogel::ArichAerogel(){
  fPosition= Hep3Vector(0,0,1400); //[mm]  Aerogel Exit
  fNorm    = Hep3Vector(0,0,1);
  fRind    = 1.05;
  fTransmissionLength= 45;         //[mm]
  fThickness = 20;                 //[mm]
  fPosition0=   fPosition - fNorm * fThickness;
  fRadius2[0] = sqr(500);                //[mm^2]
  fRadius2[1] = sqr(1300);               //[mm^2]
  fExit=-fPosition*fNorm;          //[mm]
  fEntrance= -fPosition0*fNorm;    //[mm]

  if (!gArichDetector) {
    cout << "ArichAerogel::ArichAerogel()->  gArichDetector not initilaized. Should already be initialized." << endl;
    

  } else {
    HepDouble rind=Refind(gArichDetector->Energy(1));
    const HepDouble c0=36.98; // [1/eV/mm]
    fSthc2Max= 1-1/sqr(rind);
  }  
}

ArichAerogel::~ArichAerogel(){}

HepDouble ArichAerogel::N0(){
  const HepDouble c0=36.98; // [1/eV/mm]
  const HepDouble dE=gArichDetector->Energy(1)-gArichDetector->Energy(0);
  const HepDouble epsilonmax= gArichDetector->PeakEfficiency();
  const HepDouble eta=10./17.; // parameter to acommodate additional efficiency
  return  c0 * epsilonmax * eta *dE;   

}

HepDouble ArichAerogel::NearestBorder(Hep3Vector r,Hep3Vector dir,
				    Hep3Vector &norm, HepInt &type){

  HepDouble r1=RayPlaneDistance(r,dir,fNorm,fEntrance);
  HepDouble r2=RayPlaneDistance(r,dir,fNorm,fExit);

  norm=fNorm;
  
  if ( r1>0 && r2>0) {
    type=-1;
    return min(r1,r2);
  }
  if ( r1<=0  && r2>0) {
    type=0;
    return r2;
  }
  if ( r2<=0  && r1>0) {
    type=1;
    return r1;
  }
  type=-2;
  return 0;
}


ostream&operator<<(ostream& o, ArichAerogel& d){

  return o << "Aerogel" << endl
	   << "Position  =" << d.fPosition << " mm" << endl
	   << "Direction =" << d.fNorm << endl
    	   << "Thickness =" << d.fThickness << " mm" << endl
	   << "Refind    =" << d.fRind << endl 
	   << "Trlength  =" << d.fTransmissionLength << " mm" << endl 
	   << "Inner R   =" << sqrt(d.fRadius2[0]) << " mm" << endl 
	   << "Outher R  =" << sqrt(d.fRadius2[1]) << " mm" << endl ;
}









