// -*- C++ -*-
//
// Package:     Arich
// Module:	
//
// Description: 
//
// Usage:
//      [class and public member function
//       documentation goes here]
//
// Author:      R. Pestotnik
// Created:     Thu May  1 00:00:01 EST 2003
// $Id$
//
// Revision history
//
// $Log$


#undef TESTCHRONO

#include "ArichChrono.hh"
#include <unistd.h>
#include <sys/times.h>
#include <iostream.h>
#include <string.h>

/*
class Chrono {
public:
  //-- Constructors
  // Constructors: they need a name for the Chrono that can be defined
  // both by a string or by an array of characters
  Chrono(char *s);
  //--
  // Constructor wit chrono name given by a string
  Chrono(String &s);
  //-- Destructor
  // Destructor: currently void
  ~Chrono();
  
  //-- Behavioural operations
  //   Reset counters and stored time
  void Reset();
  //--
  //   Start a counting measuring time operation
  void Start();
  //--
  //   Stop a measuring time operation
  void Stop();
  //--
  //   Inquire for the last total time measured
  clock_t LastTime();
  //--
  //   Print all information for the current ArichChrono
  void Print();
  //--
  //   Print all infos in a single line
  void PrintLine();
  //--
  //   Get the number of measurements made
  inline int TimesMeasured() const;
  //-- 
  //Get the average time for the measurements
  inline float AverageTime() const;

protected:
  //-- 
  // the Default constructor is protected to avoid copying and uncontrolled
  // creation operation
  ArichChrono();
private:
  //--
  //   ArichChrono name
  char ArichChronoName[20];
  //--
  //   Store the total number of measurements made
  int  NumberOfMeasurements;
  //--
  //   Store total user and kernel time used by the process
  clock_t    total_utime; 
  clock_t    total_stime; 
  //-- Store starting time for the current measurement
  clock_t    start_utime; 
  clock_t    start_stime; 
};
*/

//-------------- ArichChrono CLASS definitions ------------------------------------

// Static member
//--  Conversion factor from clock ticks to seconds
//    time(seconds) = rclk_tck * time(clock_ticks)
float ArichChrono::rclk_tck = 0.0; 

// Constructors: they need a name for the ArichChrono that can be defined
// both by a string or by an array of characters
  
ArichChrono::ArichChrono(char *s) {
  if( strlen(s)<14){
    strcpy(chronoName,s);
  } else {
    strncpy(chronoName,s,13);
    chronoName[13] =0;
  }
  if( rclk_tck==0.0 )   // get the relation clock ticks- seconds
    rclk_tck = 1./sysconf(_SC_CLK_TCK); 
  reset();
}
/*
ArichChrono::ArichChrono(string &s) : chronoName(s) {
  if( rclk_tck==0.0 )   // get the relation clock ticks- seconds
    rclk_tck = 1./sysconf(_SC_CLK_TCK); 
  reset();
}
*/
//Destructor: now is empty
ArichChrono::~ArichChrono(){}

// Reset operations
void ArichChrono::reset(){
  numberOfMeasurements = 0;
  total_utime = 0;
  total_stime = 0;
  start_utime = (clock_t) -1;
  start_stime = 0;
}

// Start:  Start a measuring time operation
void ArichChrono::start(){
  struct tms measuredTime;
  if(start_utime!= (clock_t)-1)
    cout<<" ArichChrono "<<chronoName<<
      " error: received a Start message without a preceding Stop one!"<<endl;
  // get the system times for the current process
  times(&measuredTime);
  // Store in object the measured times
  start_utime = measuredTime.tms_utime;
  start_stime = measuredTime.tms_stime;
}


// Stop:  Stop a measuring time operation and update internal counters
void ArichChrono::stop(){
  struct tms measuredTime;
  // test start_utime to see if there is a measurement in progress  
  if(start_utime==(clock_t) -1){
    // no measurement in progress!!!
    cout<<" ArichChrono "<<chronoName<<
      " error: received a Stop message without a preceding Start one!"<<endl;
  } else {
    // get the system times for the current process
    times(&measuredTime);
    // Evaluate elapsed times
    clock_t elapsed_time;
    // evaluate for system time
    elapsed_time = measuredTime.tms_stime - start_stime;
    total_stime += elapsed_time;
    start_stime = elapsed_time;
    // evaluate for user time
    elapsed_time = measuredTime.tms_utime - start_utime;
    total_utime += elapsed_time;
    start_utime = (clock_t) -1;
    start_stime += elapsed_time;
    // update number of measurements made
    numberOfMeasurements++;
    // after this point in total_xtime there are values relative to
    // numberOfMeasurements measures; in start_utime there is -1 (that is
    // a flag for waiting a Start message) and in start_stime there is
    // the last measured time (user+system)
  }
}


//   Print all information for the current ArichChrono
void ArichChrono::print() const {
  float  totaltime,usertime,systemtime,avtottime,avustime,avsystime;
  evaluateSeconds(totaltime,usertime,systemtime,
		  avtottime,avustime,avsystime);

  // print out
  cout<<"ArichChrono: "<<chronoName<<"     \tMeasurements :"
      <<numberOfMeasurements<<endl;
  cout<<"Total   time "<<totaltime<<" s  \tUser: "<<usertime
      <<" s \tSystem "<<systemtime<<" s"<<endl;
  if(numberOfMeasurements>0){
    cout<<"Average time "<<avtottime<<" s  \tUser: "<<avustime
	<<" s \tSystem "<<avsystime<<" s"<<endl;
  }
}

//   Print all infos in a single line
void ArichChrono::printLine() const {
  float  totaltime,usertime,systemtime,avtottime,avustime,avsystime;

  evaluateSeconds(totaltime,usertime,systemtime,
		  avtottime,avustime,avsystime);

  cout<<chronoName;
  for(int i=15-strlen(chronoName); i>0; i--)
    cout<<" ";
  cout.width(4);

  cout.setf(ios::fixed);
  //  cout.precision(4);
  cout<<numberOfMeasurements<<" \t"
      <<totaltime<<"  "<<usertime<<"  "<<systemtime<<"  "
      <<avtottime<<"  "<<avustime<<"  "<<avsystime<<endl;
  /*  cout.form("%5d %7.3f %7.3f %7.3f   %7.4f %7.4f %7.4f\n",
 	   numberOfMeasurements,
           totaltime,usertime,systemtime,
           avtottime,avustime,avsystime);
	   */
}

//    Evaluate times in seconds: private member function
void  ArichChrono::evaluateSeconds(float & totaltime, float & usertime,
			      float & systemtime,float & avtottime,
			      float & avustime, float &avsystime) const {
  // evaluate times in seconds
  totaltime = (total_utime+total_stime)*rclk_tck;
  usertime  = total_utime * rclk_tck;
  systemtime= total_stime * rclk_tck;
  if(numberOfMeasurements>0){
    // evaluate average times
    float r = 1./numberOfMeasurements;
    avtottime  = totaltime*r;
    avustime   = usertime*r;
    avsystime = systemtime*r;
  } else {
    avtottime = 0.0;
    avustime  = 0.0;
    avsystime = 0.0;
  }
}



#ifdef TESTCHRONO
main(){
  ArichChrono  outer("Outer loop");
  ArichChrono  inner("Inner loop");
  ArichChrono  medium("Medium loop");
  
  int i,j;
  outer.start();
  for(i=0; i<200; i++){
     medium.start();
     for(j=0; j<100; j++){
       inner.start();
       inner.stop();
     }
     medium.stop();
  }
  outer.stop();

  outer.print();
  medium.print();
  inner.print();  

}
#endif

