// -*- C++ -*-
//
// Package:     Arich
// Module:	
//
// Description: 
//
// Usage:
//      [class and public member function
//       documentation goes here]
//
// Author:      R. Pestotnik
// Created:     Thu May  1 00:00:01 EST 2003
// $Id$
//
// Revision history
//
// $Log$


#include "ArichConfig.hh"
#include "ArichDetector.hh"

#include <stdio.h>
#include <string>


ArichDetector::ArichDetector(){

  fPosition = Hep3Vector(0,0,1600);
  fNorm     = Hep3Vector(0,0,1);
  fbX       = Hep3Vector(1,0,0);
  fbY       = Hep3Vector(0,1,0);
  fPad      = 4.5;
  fPad      = 6;
  fNX       = 16*16 * 2;
  fNY       = fNX;
  
  fBits       = 8*sizeof(HepInt);
  fBitMapSize =  fNX*fNY/fBits;
  if (fBitMapSize * fBits < fNX*fNY ) fBitMapSize ++;
  fBitMap   = new HepInt[fBitMapSize];
  
  fCorner   = fPosition - fbX * fPad * (fNX/2) - fbY * fPad *(fNY/2) ;
  
  fRadius[0] = 500;                //[mm]
  fRadius[1] = 1300;               //[mm]
  fsigma_pad = 0.010;               // [rad]
  fEntrance      = -fPosition*fNorm;     //[mm]


  fEnergy[0]=1;  // sensitive energy window lower edge (eV)
  fEnergy[1]=5;  // sensitive energy window upper edge (eV)

  fMagneticField = 1.5;                 // 1.5T
  fWindowThickness=2;                   // [mm]
  fWindowRefind=1.5;          

  fThcMin=50e-3;
  fThcMax=500e-3;
  fWideWinSize=10;
  fNarrowWinSize=3;
  
  Clear();
  
}

ArichDetector::~ArichDetector(){
  if (fBitMap) delete fBitMap;
}

HepInt ArichDetector::Digitization( ArichPhoton *p,HepInt type){
  ArichHit hit;
  HepInt xch,ych;
  HepInt id= Position2Hit(p->R(), xch, ych );
  //cout << p->R() << Hit2Position(id,xch,ych)<< endl;
  if (id>=0) {
    if ( type && !GetBit(id)) {
      hit.Set(p->R());
      gArichHit.push_back(hit);
    }
    SetBit(id);
  }
  return id;  
}

HepDouble  ArichDetector::QEBialkali(HepDouble energy){
  // qe normalized to 1
const HepFloat qe[54]={ 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 1.00
		     ,4.00, 9.00,12.00,15.00,17.00,18.00,19.00,19.70
		     ,20.00,20.20,20.30,20.40,20.30,20.20,20.10,20.00
		     ,19.70,19.00,18.00,17.00,16.00,14.50,13.50,12.00
		     ,11.00,10.00,9.00, 7.00, 5.00, 4.00, 3.50, 3.00
		     ,2.50 ,2.00 ,1.70 ,1.00 ,0.70 ,0.40 ,0.20 ,0.10
		     , 0.08, 0.04, 0.02, 0.01, 0.00, 0.00};

 HepInt ich=(int)( (1239.85/energy-185)/10.);
 if (ich<0 || ich >=54)  return 0;
 return qe[ich]/20.40;  
 
}



void   ArichDetector::SetBit(HepInt bit){
  //cout << "Sbit " << bit << endl;
  //if (GetBit(bit)) cout << "Already set" << endl ;
  fBitMap[bit/fBits] |= (1 << (bit % fBits));
  //if (!GetBit(bit)) cout << "Error" << endl ;
}


void ArichDetector::Clear(){
  if (fBitMap) memset(fBitMap,0,fBitMapSize*sizeof(HepInt));    
}




HepInt ArichDetector::HitReconstruction(){
  ArichHit hit;
  Hep3Vector r;
  HepInt ix,iy;
  /*  
  for (int i=0;i<fNY*fNX;i++){
    gTimer[6]->start();
	  
    if (GetBit(i)) {

      hit.Set(Hit2Position(i,ix,iy));
      gArichHit.push_back(hit);
    

    }
    gTimer[6]->stop();
  }
  */
  HepInt bit;
  HepInt *mem=fBitMap;
  for (int i=0;i<fBitMapSize;i++){
    //gTimer[5]->start();
    if (*mem){
      bit=1;
      for (int j=0;j<fBits;j++){
	if (*mem & bit){
	  hit.Set(Hit2Position(i*fBits + j,ix,iy));
	  gArichHit.push_back(hit);
	} 
	bit<<=  1;
      }
    }
    mem++;
    //gTimer[5]->stop();
  }

  return gArichHit.size();
}

HepInt ArichDetector::Position2Hit(Hep3Vector r, HepInt &xch, HepInt &ych ){
  Hep3Vector r0=r-fCorner;
  HepDouble x= r0*fbX;
  if (x<0) return -1;
  
  HepDouble y= r0*fbY;
  if (y<0) return -1;
  
  xch = (int) (x/fPad);
  if (xch>= fNX) return -1;
  
  ych = (int) (y/fPad);
  if (ych>= fNY) return -1;
  
  return (ych * fNX + xch);
}

Hep3Vector ArichDetector::Hit2Position(HepInt ch, HepInt &xch, HepInt &ych){

  ych= ch / fNX;
  xch= ch - ych * fNX;
  return fCorner+(xch+0.5)*fPad*fbX + (ych+0.5)*fPad*fbY;
}

ostream&operator<<(ostream& o, ArichDetector& d){
  return o << "Detector" << endl
	   << "Position  =" << d.fPosition << " mm" << endl
	   << "Direction =" << d.fNorm << endl
	   << "PadSize   =" << d.fPad << " mm" << endl
	   << "Sigma Pad =" << d.fsigma_pad << endl 
	   << "N channels x  =" << d.fNX  << endl 
	   << "N channels y  =" << d.fNY  << endl 
	   << "Inner R   =" << d.fRadius[0] << " mm" << endl 
	   << "Outer R  =" << d.fRadius[1] << " mm" << endl; 

}


