// -*- C++ -*-
//
// Package:     Arich
// Module:	
//
// Description: 
//
// Usage:
//      [class and public member function
//       documentation goes here]
//
// Author:      R. Pestotnik
// Created:     Thu May  1 00:00:01 EST 2003
// $Id$
//
// Revision history
//
// $Log$


#include "ArichGeo.hh"
#include "ArichAerogel.hh"
#include "ArichDetector.hh"
#include "ArichConfig.hh"

Hep3Vector ArichGeo::RayTrace(Hep3Vector r, Hep3Vector dir){
  Hep3Vector r0,dir0;

  r0   = RayToPlane(r,dir,gArichAerogel->fNorm, gArichAerogel->fExit);  
  dir0 = Refraction(dir,gArichAerogel->fNorm,gArichAerogel->Refind());  
  r0   = RayToPlane(r0,dir0,gArichDetector->fNorm, gArichDetector->fEntrance);

  return r0;
}

HepDouble  ArichGeo::RayTubeDistance(Hep3Vector r, Hep3Vector dir,  Hep3Vector p, Hep3Vector norm,HepDouble d){
  HepDouble sndir=norm*dir;
  Hep3Vector r1= Perpendicular(r,norm);
  Hep3Vector p1= Perpendicular(p,norm);
  Hep3Vector dir1= Perpendicular(dir,norm);


  HepDouble aa=2*dir1.mag2();
  HepDouble bb=-2*(p1-r1)*dir1;
  HepDouble cc=(p1-r1).mag2();
  HepDouble xx=bb*bb-2*aa*cc;
  if (xx==0) return -bb/aa;
  if (xx>0){
    xx=sqrt(xx);
    HepDouble r1=(-bb-xx)/aa;
    HepDouble r2=(-bb+xx)/aa;
    if ( r1>=0 && r2>=0)  return min(r1,r2);
    if ( r1<0  && r2>=0)  return r2;
    if ( r2<0  && r1>=0)  return r1;
  };
  
  cout << "Ray does not intersect the Tube" << endl;  
  return -1;

}


HepInt ArichGeo::ReflectionRefraction(Hep3Vector &dir, Hep3Vector &pol, Hep3Vector &norm, HepDouble refind){
  // refraction/reflection on the borders between media 1 and 2   (1->2)
  // dir, pol direction and polarization [input/output]
  // norm     reflaction plane [input]
  // refind   refractive index of the medium n(2)/n(1)
  
  // iproc == 1 reflection
  // iproc == 0 refraction 
  Hep3Vector b1=dir.cross(norm).unit();
  Hep3Vector b2=dir.cross(b1);
  Hep3Vector b3=norm.cross(b1);

  HepDouble polf[2]={ pol*b1, pol*b2 };

  HepDouble Evzp=sqr(polf[0]);
  HepDouble Eprv=sqr(polf[1]);
  
  HepDouble dir3= norm * dir;
  HepDouble sthl=sqrt(1-dir3*dir3) / refind;
  HepDouble transmission=0, proba=1;
  HepDouble rvzp,tvzp,rprav,tprav;
  HepDouble cthl;    
  if (sthl<1){
    cthl= sqrt(1-sthl*sthl); // cos of refracted angle
    HepDouble cth=fabs(dir3);
    HepDouble sth=sqrt(1-sqr(cth));
    
    HepDouble th = acos(cth);
    HepDouble thl= asin(sthl);
    //cout << "ReflectionRefraction " << refind << " " << th << " " << thl << endl;      
    if (cth!=1.0){
      rvzp  = -sin(th-thl)/sin(th+thl);
      tvzp  = 2*cth*sthl/sin(th+thl);
      rprav = tan(th-thl)/tan(th+thl);
      tprav = 2*cth*sthl/sin(th+thl)/cos(th-thl);
    } else {
      rvzp  = (1-refind)/(1+refind);
      tvzp  = sqrt(1-sqr(rvzp));
      rprav = rvzp;
      tprav = tvzp;

    }
    transmission=Evzp*(1-sqr(rvzp))+Eprv*(1-sqr(rprav) );
    //cout <<  "RRX " << dir*pol << "d="<< dir.mag() << "p="<< pol.mag() << " Trans="<<transmission<< " "<< Evzp<<" "<< rvzp  <<" "<< Eprv  << " "<< rprav  <<   endl;
    proba   = ArichRand();
  }
  HepInt iproc=  (proba > transmission);
  if (iproc) {
    // reflection 
    polf[0]*=rprav;
    polf[1]*=rvzp;

    dir = (dir-2*dir3*norm);
    if (sthl>=1) iproc++;
  } else {
    // refraction
    polf[0]*=tprav;
    polf[1]*=tvzp;
    dir = (sthl*b3+cthl*norm);
  }

  b2= dir.cross(b1);
  pol=polf[0]*b1+polf[1]*b2;
  pol=pol.unit();
  return iproc;
}





