// -*- C++ -*-
//
// Package:     Arich
// Module:	
//
// Description: 
//
// Usage:
//      [class and public member function
//       documentation goes here]
//
// Author:      R. Pestotnik
// Created:     Thu May  1 00:00:01 EST 2003
// $Id$
//
// Revision history
//
// $Log$


#include <stdlib.h>
#include <math.h>
#include <iostream>
#include "ArichUtil.hh"

using namespace std;


#define SQPI 2.50662827463


int   Arich::RandPoisson(float xm  ){
 
  // Returns as a floating-point number an integer value that is a random
  // deviation drawn from a Poisson distribution of mean xm, using flat()
  // as a source of uniform random numbers.
  // (Adapted from Numerical Recipes in C)
  //  works for xm<12
    
      register float em;
      float t, y;
      static float g,om = -1.0;
    
    
      if( xm == -1 ) return 0;
      if( xm < 12.0 ) {
        if( xm != om ) {
          om= xm;
          g = exp(-xm);
        }
        em = -1;
        t = 1.0;
        do {
          em += 1.0;
          t *= Arich::Rand();
        } while( t > g );
      } else {
	return -1;
      }
      return (int) em;
}



float   Arich::RandGauss( float m, float s){
  
  /*     normal random variate generator */
  /*     mean m, standard deviation s */
  float x1, x2, w, y1;
  static float y2=0.0;
  if (y2!=0.0){
    y1=y2;
    y2=0.0;
  } else {

    do{
      x1 = 2.0 * Arich::Rand() - 1.0;
      x2 = 2.0 * Arich::Rand() - 1.0;
      w = x1 * x1 + x2 * x2;
    } while ( w >= 1.0 );
  
    w = sqrt( (-2.0 * log( w ) ) / w );
    y1 = x1 * w;
    y2 = x2 * w;
  }

  //cout << "G"; 
  return (m + y1 * s);
  
}




double Arich::Algama(double x){

  
  const double p1[7]={+3.842873656745991E+0, +5.270689375300983E+1,
		      +5.558404572351531E+1, -2.151351357372570E+2,
		      -2.458726172229242E+2, -5.750089360304123E+1,
		      -2.335909894951284E+0 };

  const double  q1[7]= {+1.000000000000000E+0, +3.373304790707074E+1,
			+1.938778403437713E+2, +3.088295497342428E+2,
			+1.500683906489095E+2, +2.010685134433395E+1,
			+4.571742028250299E-1};
  
  const double  p2[7]={ +4.8740201396838636E+0, +2.4884525168574076E+2,
			+2.1797366058895915E+3, +3.7975124011525118E+3,
			-1.9778070769841646E+3, -3.6929834005591282E+3,
			-5.6017773537803877E+2};
  
  const double  q2[7]={ +1.0000000000000000E+0, +9.5099917418208938E+1,
			+1.5612045277928635E+3, +7.2340087928948071E+3,
			+1.0459576594058959E+4, +4.1699415153200231E+3,
			+2.7678583623804101E+2};
  
  const double  p3[7]={ -6.880624009459425E+3, -4.306996981957098E+5,
			-4.750459465343956E+6, -2.942344593032234E+6,
			+3.632180493154257E+7, -3.356778281454576E+6,
			-2.480436948828593E+7};
  const double  q3[7]={ +1.000000000000000E+0, -1.421682983965146E+3,
			-1.555289028085353E+5, -3.415251710801107E+6,
			-2.096962325580444E+7, -3.454417509334395E+7,
			-9.160558286371317E+6};
  const double  c[5]={  1.1224921356561E-1,  7.9591692961204E-2,
			-1.7087794611020E-3,  9.1893853320467E-1,
			1.3469905627879E+0};
  

  const double z1=1;
  const double hf=z1/2;
  const double hf1=1+hf;
  
  double ap,aq,h;
 
  if (x<0){
    h=0;
    cout << "NON-POSITIVE ARGUMENT  X =" << x << endl;
  } else if (x ==1 || x==2) {
    h=0;
  } else if (x <= hf) {
    double y=x+1;
    ap=p1[0];
    aq=q1[0];
    for (int i=1; i<7;i++ ){
      ap=p1[i]+y*ap;
      aq=q1[i]+y*aq;
    }
    h=-log(x)+x*ap/aq;
  } else if  (x <= hf1) {
    ap=p1[0];
    aq=q1[0];
    for (int i=1; i<7; i++ ){
      ap=p1[i]+x*ap;
      aq=q1[i]+x*aq;
    }
    h=(x-1)*ap/aq;
  } else if  (x <= 4) {
    ap=p2[0];
    aq=q2[0];
    for (int i=1; i<7; i++ ){
      ap=p2[i]+x*ap;
      aq=q2[i]+x*aq;
    }
    h=(x-2)*ap/aq;
  } else if  (x <= 12) {
    ap=p3[0];
    aq=q3[0];
    for (int i=1; i<7; i++ ){
      ap=p3[i]+x*ap;
      aq=q3[i]+x*aq;
    }
    h=ap/aq;
  } else {
    double y= 1/x/x;
    h= (x-hf)*log(x)-x+c[3]+(c[0]+y*(c[1]+y*c[2]))/((c[4]+y)*x);
  }
  return h;
}

double Arich::PoisLog(double mean, double  n){

  if (mean==0){
    if (n==0) return 0;
    else return -1e10;
    
  } else {
    // double x=n+1;
    //    return (n*log(mean)-mean-Arich::Algama(x));
    //    for different particle hypotheses Algama cancels, since n=sfot is the same for all the hypotheses
    return (n*log(mean)-mean); 
  }
};



void Arich::LogLikelihood(float x,float sig,float wt,int n, double *x0,
			double* stfs, float stfb,float a,double *plkh,double &sfot){

  // linear background assumed

  // X, SIG, WT           ! cerenkov angle, sigma, weight [in]
  // N                    ! number of hypotheses [in]
  // X0(0:N-1)            ! cerenkov angles for other,e,mu,pi,K,p [in]
  // STFS(0:N-1)          ! expected number of cerenkov photons [in]
  // STFB                 ! expected number of background photons [in]
  // A                    ! background normalisation constant [in]
  // PLKH(0:N-1)          ! logLikelihoods (spatial part) [in,out]
  // SFOT                 ! number of photons (weighted sum) [in,out]
  
  float z,f,g,konst,b,fb;

  if (x<=0) return;
  if (stfb==0){
    konst=log(SQPI*sig);
    for (int k=0; k< n;k++){
      if (stfs[k]>0){
	z=(x-x0[k])/sig;
	f=-(z*z/2.+konst);
	plkh[k]+=wt*f;
      }
    }
  } else {
    b= a*x;
    fb=log(b);
    for (int k=0; k< n;k++){
      if (stfs[k]>0){
	z=(x-x0[k])/sig;
	if (fabs(z)<3) {
	  g= exp(-z*z/2)/SQPI/sig;
	  f=log((stfs[k]*g+stfb*b)/(stfs[k]+stfb));
	} else {
	  f=log(stfb*b/(stfs[k]+stfb));
	}
	plkh[k]+=wt*f;
      }
    }
  }

  sfot=sfot+wt;  
  return;
}

 float   Arich::RandBackground(float max){
  // The parametrization of the background
  // based on the position of the track
  float thc;
  do   thc=Arich::Rand()*max; while ( Arich::Rand() > thc/max);
  return thc; 
}
 
