// -*- C++ -*-
//
// Package:     Arich
// Module:	
//
// Description: 
//
// Usage:
//      [class and public member function
//       documentation goes here]
//
// Author:      R. Pestotnik
// Created:     Thu May  1 00:00:01 EST 2003
// $Id$
//
// Revision history
//
// $Log$


#include <stdlib.h>
#include <CLHEP/config/CLHEP.h>
#include <CLHEP/Vector/ThreeVector.h>

#define  Min(x,y,s) (s*x<s*y)?x:y
#define  sign(x) ((x>0)*2-1)

HepDouble HelixWithTube(HepDouble *rc,HepDouble r,HepDouble cut,HepDouble omega,HepDouble *p){

  // rc[2]  ring center
  // r ring radius
  // cut  radius

  HepDouble c= cut*cut - rc[1]*rc[1]-rc[0]*rc[0]-r*r;
  c=c/(2*r*r);
  HepDouble a= rc[0]/r;
  HepDouble b= rc[1]/r;
  // fa*y*y + fb*y + fc = 0
  HepDouble fa=a*a+b*b;
  HepDouble fc=(c*c-a*a)/fa;
  HepDouble fb=-2*c*b/fa;
  HepDouble d = fb*fb-4*fc;

  if (d<0) return 0;
  d=sqrt(d);
  HepDouble py[2]={(-fb+d)*0.5,(-fb-d)*0.5};
  HepDouble px[2]={sqrt(1-p[0]*p[0]),sqrt(1-p[1]*p[1])};

  int k=0;
  for (int i=0;i<2;i++) 
    for (int j=0;j<2;j++) {
      if (fabs(a*px[j]*(1.-2*j)+b*py[i]-c) < 1.0E-15)
	if (k<2) {
	  p[k]=atan2(py[i],px[j]);
	  k++;
	} else cout << "HelixWithTube has too many solutions!" << endl;
    }

  return 1;

}

HepInt BzTrack(HepDouble mass,HepDouble charge,
	       Hep3Vector r, Hep3Vector dir, HepDouble momentum,
	       HepDouble Bz, HepDouble zout, HepDouble rcut,
	       Hep3Vector &r0, Hep3Vector &dir0){
  /*
     CALCULATES INTERSECTION OF A CHARGED PARTICLE TRAJECTORY
     WITH A PLANE PERPENDICULAR TO THE CONSTANT MAGNETIC FIELD

      mass     PARTICLE MASS [GeV/c2]     
      charge   PARTICLE CHARGE  +1 /-1
      r        PARTICLE POSITION [mm]
      dir      PARTICLE DIRECTION
      p        PARTICLE MOMENTUM [GeV/c] 
      bz       MAGNETIC FIELD   [T]
      zout     Z OF THE PLANE   [mm]      

      return   new position
  */

  const HepFloat c=3E8;

  HepDouble BetaGama = momentum/mass;
  HepDouble beta     = BetaGama/sqrt(1+BetaGama*BetaGama);
  HepDouble gama     = BetaGama / beta;  
  HepDouble vl       = dir.z()*beta*c;
  HepDouble dirt     = sqrt(dir.x()*dir.x()+dir.y()*dir.y());

  HepDouble rinc= dirt*momentum/(0.3*charge*Bz)*1000; //  0.3= (eV*10**9)/ (3*10**8 m/s)
  HepDouble t = (zout-r.z()) / vl /1000;
  if (t<0) return 0;
  HepDouble fc= rinc/dirt;
  HepDouble rc[2]={r.x()+dir.y()*fc,r.y()-dir.x()*fc};
  HepDouble omega=-charge*Bz*0.3*c/mass/gama; //  0.3= (eV*10**9)/ (3*10**8 m/s)
  rinc = fabs(rinc);
  HepDouble fi0=atan2((r.y()-rc[1]),r.x()-rc[0]);
  
  HepDouble alpha=omega*t+fi0;
  HepDouble cfi=cos(alpha);
  HepDouble sfi=sin(alpha);

  r0 = Hep3Vector( rc[0] + cfi*rinc , rc[1] + sfi*rinc , zout);
  dir0 =Hep3Vector(dir.x()*cfi-dir.y()*sfi, dir.x()*sfi+dir.y()*cfi,dir.z() );

  HepDouble p[2];
  if (HelixWithTube(rc,rinc,rcut,omega,p)) {
    if (omega<0) {p[0]*=-1.; p[1]*=-1.; fi0*=-1.; alpha*=-1.;};
    for (int i=0;i<2;i++) 
      if (alpha>p[i] && p[i]>fi0) return 0;
  }

  return 1;
}





#ifdef MAIN
#include <stdio.h>

int testhelix(int argc, char ** argv){

  HepDouble p_mass[5]={.511E-3,105.66E-3,139.57E-3,493.68E-3,938.27E-3};
  HepDouble Momentum=1.;
  HepDouble rcut=1500;
  HepDouble Charge=-1;
  HepDouble Bz=1.5;
  Hep3Vector Position(100,-250,0);
  Hep3Vector Direction= Hep3Vector(0.1,0.3,0.9).unit();

  Hep3Vector v,dir;
  HepInt imax=1000;
  for (int i=0;i<imax;i++){
    HepDouble zout=10000.*i/imax;
    BzTrack(p_mass[2],Charge,Position,Direction,Momentum,Bz,zout,rcut,v,dir);
    cout << v.x() << " " << v.y() << " "<< v.z() << endl;
    
  }
   
  return 0;
}

FILE *fin;

HepInt GetEvent(){


  static int first=1;
  if (!fin && first) {
    fin=fopen("/u/argus/bracko/mcdata/mc_gen_0.dat","r");
    first=0;
  } else  if (!fin && !first) return -1;
  int j=0;
  HepFloat r[3],p[3];
  HepInt code,neve,ntra;
  int ndim=400;
  char line[ndim];
  int nb1=0;
  do {
    if (fgets(line,ndim,fin)==NULL) break;
    nb1=sscanf(line,"* %d%d",&neve,&ntra);
    //cout << nb1 << " " << line << endl;
    if (nb1==2){
      
      for (int i=0;i<ntra;i++){
	if (fgets(line,ndim,fin)==NULL) break;
	int nb2=sscanf(line,"%d%f%f%f%f%f%f",&code,&p[0],&p[1],&p[2],&r[0],&r[1],&r[2]);

	if (nb2==7) {
	  Hep3Vector pos(r[1],r[0],r[2]);
	  Hep3Vector mom(p[1],p[0],p[2]);
	  Hep3Vector dir    =mom.unit();
	  HepDouble momentum=mom.mag();

 //         Hep3Vector r1,dir1;
//	  BzTrack(105.66E-3,1,pos,dir,momentum,1.5,1600,1300,r1,dir1);

//	  cout << r1.x() << " " << r1.y() << " "<< r1.z() << endl;

//	  cout << i << pos << mom << endl;
	}
      }
//      cout << "Event " << neve << " ntracks " << ntra <<endl;
    }
   
  } while (nb1!=2);
  if (feof(fin)) fclose(fin);

  return neve;
}



int main(int argc, char ** argv){
//    for (int i=0;i<100;i++) GetEvent();

  return 0;
}
#endif





