#ifndef _PID_Arich_hh_
#define _PID_Arich_hh_

#include <CLHEP/config/CLHEP.h>
#include <CLHEP/Vector/ThreeVector.h>
#include <CLHEP/Vector/LorentzVector.h>
#include "CLHEP/Random/Randomize.h"
#include "ArichChrono.hh"
#include "ArichUtil.hh"
#include <stdio.h>


class PID_Arich{
public:
  PID_Arich();
  ~PID_Arich();


  HepInt init(HepInt debug_level=0, HepInt mode=1);
  HepInt Run();
  HepInt End();
  HepInt GetEvent();

  HepInt pid( HepLorentzVector p, Hep3Vector p_measured, Hep3Vector x );
  // set the 4-momentum, measured momentum and position of the
  // particle

  HepDouble prob( HepInt type ); // probability assuming e,mu,pi,K,p


private:

  HepInt fFastPID;
  
  HepInt nPhotons;
  HepInt maxPhotons;

  HepInt nHits;
  HepInt maxHits;
  

  HepInt fRichTracks;    // number of tracks for which the photons were generated
  HepInt fRecoTracks;    // number of tracks for which the likelihoods are calculated
  HepInt fTrackedTracks; // number of tracks in RICH
  HepInt fAllTracks;      // number of all tracks
  
  HepInt nTracks;
  HepInt maxTracks;

  HepInt fRichEvents;
  HepInt fAllEvents;
  
  
  
  HepInt GetTracks();

  HepInt GetTrack(HepInt &lund,Hep3Vector &r, Hep3Vector &p);

  HepInt Clear();
  
  HepInt FastPhotonGeneration();
  
  HepInt ParticleTracking();
  HepInt PhotonGeneration();
  HepInt PhotonTracking();
  HepInt Digitization();
  HepInt HitReconstruction();
  
  HepInt SignalEstimate();
  HepInt CherenkovAngleReconstruction();
  HepInt BackgroundEstimate();
  HepInt BackgroundFitEstimate();

  HepInt Likelihood();
  
  HepInt FastParticleId();
  HepInt FastPid(float p, int id, double *plkh, double &genfot, double &sfot, double &stfb, double *stfs, double *thce);
  
  HepFloat  FastBackground();
    
  void Normalize(HepDouble *plkh, HepInt nhyp, HepInt norm, HepInt &idp);


  
  FILE *fin;
  HepInt nfile;

};

#endif










