// -*- C++ -*-
//
// Package:     Arich
// Module:	/
//
// Description: This file includes the example main program which 
//              
// Usage:
//      <exename> <arg1> <arg2>
//      <arg1>     debug mode 
//      <arg2>     number of events to process
//      <arg3>     simulation mode
//
// debug level
// 0 - nothing
// 1 - histogramming
// 2 - event output
// 3 - track output
// 4 - photon output
//
// simulation mode
// 0 Full PID - (particle tracking, hit generation,
//               ch.angle reconstruction, likelihood calculation)
// 1 Fast PID
//
// Author:      R. Pestotnik
// Created:     Thu May  1 00:00:01 EST 2003
// $Id$
//
// Revision history
//
// $Log$


#include "ArichConfig.hh"
#include "ArichUtil.hh"
#include "PID_Arich.hh"
#include "ArichDetector.hh"

/*-----------------------------------------    HBook ------------- */
extern "C" {
#include "hbooklib.h"
}
#define PAWC_SIZE 500000

class NT {
public:
  int id;
  float x,y,z,sx,sy,sz,p,sfot,stfb,ngfot,rl[5],np[5],thc[5],ac[5];
  NT(){};
  ~NT(){};
  void Print(int i);
};

void NT::Print(int idx){
  printf("%d (%f %f %f) (%f %f %f) %f %f %f\n",id,sx,sy,sz,x,y,z,p,sfot,ngfot,stfb );
  for ( int i=0;i<5;i++) printf("[%d] %d. %f %f %f %f\n",idx,i,ac[i],rl[i],np[i],thc[i]); 
  return;
}
static NT nt;

int hbookinit(char *filename)
{
  char *opt="NX";
  int record_size=1024;
  int istat;
  char *ldir="EXP";
  int Ntmp = 0;
  


  Hlimit(PAWC_SIZE);
  
  
  Hropen(1, ldir, filename, opt, &record_size, &istat);   
  if (istat) {
    cout << "Error in opening file " <<  filename << endl;
    return -1;
  }
  
  return 0;
}

int hbookend()
{
  int i,id;
  int icycle=0,noent=0;
  Hrout(0,&icycle," ");
  Hrend("EXP");
  return 0;
}

int histoinit(){

  // ngen
  Hbook1(100,"All  Tracks/event", 30, -0.5, 29.5, 0.);
  Hbook1(101,"RICH Tracks/event", 10, -0.5, 9.5, 0.);
  Hbook1(102,"MC truth", 10, -5, 5, 0.);
  Hbook1(103,"Momentum",  100, 0 , 5, 0.);

  Hbook2(104,"MC truth vs Momentum",  5, -0.5, 4.5,100, 0 , 10, 0.);
  
 
  Hbook1(105,"Photon energy",  40, gArichDetector->fEnergy[0] , gArichDetector->fEnergy[1], 0.);
  Hbook2(106,"Track inc.angle vs Radius",40, 0 , 1500, 40 , 0, M_PI/2, 0.);
  Hbook1(107,"Momentum RECO>0",  100, 0 , 5, 0.);
  
  Hbprof(110,"Ngenfot vs p(e)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(111,"Ngenfot vs p(mu)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(112,"Ngenfot vs p(pi)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(113,"Ngenfot vs p(K)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(114,"Ngenfot vs p(p)" ,40, 0 , 4, -0.5, 39.5, " ");  

  Hbprof(120,"Radiation path vs p(e)" ,40, 0 , 5, -0.5, 39.5, " ");
  Hbprof(121,"Radiation path vs p(mu)",40, 0 , 5, -0.5, 39.5, " ");
  Hbprof(122,"Radiation path vs p(pi)",40, 0 , 5, -0.5, 39.5, " ");
  Hbprof(123,"Radiation path vs p(K)" ,40, 0 , 5, -0.5, 39.5, " ");
  Hbprof(124,"Radiation path vs p(p)" ,40, 0 , 5, -0.5, 39.5, " ");  

  Hbprof(130,"Sfot vs p(e)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(131,"Sfot vs p(mu)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(132,"Sfot vs p(pi)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(133,"Sfot vs p(K)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(134,"Sfot vs p(p)" ,40, 0 , 4, -0.5, 39.5, " ");  

  Hbprof(140,"stfs vs p(e)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(141,"stfs vs p(mu)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(142,"stfs vs p(pi)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(143,"stfs vs p(K)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(144,"stfs vs p(p)" ,40, 0 , 4, -0.5, 39.5, " ");

  Hbprof(150,"thcm vs p(e)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(151,"thcm vs p(mu)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(152,"thcm vs p(pi)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(153,"thcm vs p(K)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(154,"thcm vs p(p)" ,40, 0 , 4, -0.5, 39.5, " ");

  Hbprof(160,"thc vs p(e)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(161,"thc vs p(mu)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(162,"thc vs p(pi)",40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(163,"thc vs p(K)" ,40, 0 , 4, -0.5, 39.5, " ");
  Hbprof(164,"thc vs p(p)" ,40, 0 , 4, -0.5, 39.5, " ");


  Hbook2(170,"thcm vs p(e)" ,10, 0 , 4, 80,0., .4, 0);
  Hbook2(171,"thcm vs p(mu)",10, 0 , 4, 80,0., .4, 0);
  Hbook2(172,"thcm vs p(pi)",10, 0 , 4, 80,0., .4, 0);
  Hbook2(173,"thcm vs p(K)" ,10, 0 , 4, 80,0., .4, 0);
  Hbook2(174,"thcm vs p(p)" ,10, 0 , 4, 80,0., .4, 0);
  // nhit
  HepInt nchx=gArichDetector->fNX;
  HepInt nchy=gArichDetector->fNY;
  HepDouble dx=gArichDetector->fPad;
  
  Hbook2(200,"Photons at the detector plane" , nchx/2, -nchx*dx/2, nchx*dx/2,
	 nchy/2, -nchy*dx/2 , nchy*dx/2, 0.);


  Hbook2(201,"Hits on the detector" , nchx/2, -nchx*dx/2, nchx*dx/2,
 	 nchy/2, -nchy*dx/2 , nchy*dx/2, 0.);

  
  // hits 
  Hbook1(301,"Cherenkov radius" , nchx, 0 , nchx*dx/2, 0.);


  // reco
  Hbook2(400,"Cherenkov space" , 100, -0.5 , 0.5,100, -0.5 , 0.5, 0.);
  Hbook1(401,"Cherenkov angle" , 100, 0 , 0.5, 0.);
  Hbook1(403,"Background shape" , 100, 0 , 0.5, 0.);
  Hbook1(402,"Cherenkov angle generated" , 100, 0 , 0.5, 0.);


  Hbook2(500,"Reco vs gen thc" , 100, 0, 0.5,100, 0 , 0.5, 0.);
  
  for (int i=0; i<10000;i++){
    Hfill(403,Arich::RandBackground(0.5),0,1);
  }
  
  Hbnt(1,"tracks"," ");
  // Hbname(1,"tracks",(int *) &(nt.id),"id:I,sx,sy,sz,p,x,y,z,rle,rlmu,rlpi,rlk,rlp");
   Hbname(1,"tracks",(int *) &(nt.id),"id:I,r(3),dir(3),p,sfot,stfb,gfot,rl(5),np(5),thc(5),ac(5)");
  return 0;  
}

int gArichEve=0;

int userinit(){
  hbookinit("arichlkh.hbook");
  histoinit();
  return 0;
}

int userevent(){

  if (ArichDebugLevel==0) return 0;
  // tracks
  Hfill(100,gArichParticle.size(),0,1);
  for (int i=0;i<gArichParticle.size();i++){


    HepInt id   = gArichParticle[i].MCIdentity();
    HepDouble q = gArichParticle[i].fCharge;
    HepDouble p = gArichParticle[i].fMomentum;
    Hfill(102,q*(id+0.5),0,1 );
    Hfill(103,p,0,1 );
    Hfill(104,id,p,1 );

    if (gArichParticle[i].fTracked<= 0) continue;

    // ntuple stuff
    ArichParticle *t=&gArichParticle[i];
    
    nt.id=t->MCIdentity();
    nt.sx=t->fDirection.x(); 
    nt.sy=t->fDirection.y(); 
    nt.sz=t->fDirection.z();
    nt.x=t->fAeroIn.x(); 
    nt.y=t->fAeroIn.y(); 
    nt.z=t->fAeroIn.z();

    nt.p=t->fMomentum*t->fCharge;
    nt.sfot=t->sfot;
    nt.stfb=t->stfb;
    nt.ngfot=t->fNgenPhot;

    if (t->fRecoFlag != 0.0){
      Hfill(107,p,0,1 );
      
      for (int g=0;g<5;g++){
	nt.rl[g]=t->plkh[g];
	nt.np[g]=t->stfs[g];
	nt.thc[g]=t->thc[g];
      
	nt.ac[g]=t->acceptance[g];
	//	if  ( isinf(nt.rl[g]) ) nt.rl[g]=0; 
	// if  ( isnan(nt.rl[g]) ) nt.rl[g]=0; 
		
      }
      //nt.Print(i);
      Hfnt(1);

    } else {
      for (int g=0;g<5;g++){
	nt.np[g]=0;
	nt.thc[g]=0;
	nt.rl[g]=0;
	nt.ac[g]=0;
      }      
    }
    
    HepDouble ax = gArichParticle[i].fPosition.x();
    HepDouble ay = gArichParticle[i].fPosition.y();
    
    // HepDouble dx = gArichParticle[i].fVirtualCenter.x();
    // HepDouble dy = gArichParticle[i].fVirtualCenter.y();
    //  HepDouble dx = gArichParticle[i].fAeroOut.x();
    //  HepDouble dy = gArichParticle[i].fAeroOut.y();
    
     HepDouble dx = gArichParticle[i].fDetector.x();
     HepDouble dy = gArichParticle[i].fDetector.y();
    HepInt ntra=0;
    Hfill(106,sqrt(sqr(dx)+sqr(dy)), acos(gArichParticle[i].fMeanEDir.z()), 1. );

    Hfill(110+id,p,gArichParticle[i].fNgenPhot,1 );
    Hfill(130+id,p,gArichParticle[i].sfot,1 );
    Hfill(140+id,p,gArichParticle[i].stfs[id],1 );
    Hfill(160+id,p,gArichParticle[i].thc[id],1 );
    Hfill(120+id,p,gArichParticle[i].fPath,1 );

    if (gArichParticle[i].fNgenPhot>0) {
      ntra++;
      if (gArichEve< 20 ) Hfill(200,dx,dy,1);
      if ( !gArichPhoton.size() ) continue;
      
      ArichPhoton *h=&gArichPhoton[gArichParticle[i].fPhoton];
      for (int j=0;j < gArichParticle[i].fNgenPhot;j++){
	if (h->fTracked>0){
	  HepDouble th_cer;
	  HepDouble fi_cer;
	  gArichParticle[i].CerenkovAngle(h->fPosition, th_cer, fi_cer);
	  
	  //  cout << h << th_cer << endl;
	}
	h++;
      }
      
    }
    Hfill(101,ntra,0,1);
   
    for (int i=0;i<gArichHit.size();i++){
      HepDouble x   = gArichHit[i].fPosition.x() - dx;
      HepDouble y   = gArichHit[i].fPosition.y() - dy;
      Hfill(301,sqrt(sqr(x)+sqr(y)),0,1 );    
    }
  }

  // photons
  for (int i=0;i<gArichPhoton.size();i++){
    if (gArichPhoton[i].fPath<=0) continue;
    HepDouble x   = gArichPhoton[i].fPosition.x();
    HepDouble y   = gArichPhoton[i].fPosition.y();
    if (gArichEve< 20 ) Hfill(200,x,y,1 );
    Hfill(105,gArichPhoton[i].fEnergy,0,1);
    Hfill(402,gArichPhoton[i].fThc,y,1 );
  }

  // hits
  for (int i=0;i<gArichHit.size();i++){
    HepDouble x   = gArichHit[i].fPosition.x();
    HepDouble y   = gArichHit[i].fPosition.y();
    Hfill(201,x,y,1 );
  }

  HepInt siz=gArichHit.size();
  // hittab
  for (int i=0;i<gArichHitTab.size();i++){
    HepDouble theta   = gArichHitTab[i].theta();
    HepDouble fi      = gArichHitTab[i].phi();
    ArichParticle *tr =  &gArichParticle[ gArichHitTab[i].fTrack];
    HepDouble p = tr->fMomentum;
    HepInt id   = tr->MCIdentity();
    Hfill(401,theta,0,1 );
    Hfill(400,theta*cos(fi),theta*sin(fi),1 );
    Hfill(150+id,p,theta,1 );    
    Hfill(170+id,p,theta,1 );

	
    
  }
  
  
  return 0;
}


int userend(){
  hbookend();
  return 0; 
}


int main(int argc, char ** argv){
  

  HepInt debug_level=1;
  if (argc>1) debug_level=atoi(argv[1]);
  HepInt n=-1;
  if (argc>2) n=atoi(argv[2]);
  HepInt mode=0;
  if (argc>3) mode=atoi(argv[3]);

  PID_Arich *gmain= new PID_Arich();
  gmain->init(debug_level, mode);
  userinit();

  // EVENT LOOP ************************************************
  int ieve=0;
  while (ieve<n || n<0){
    ieve++;
    gArichEve=ieve;
    HepInt ier=gmain->GetEvent();
    
    if (ier<0) break;
    gmain->Run();
    userevent();
    if (ieve%1000 == 0) cout <<  " Processed events " << ieve << endl;

  }

  // END *******************************************************
  gmain->End();
  userend();
  cout<< "Program Finished" << endl;  
  return 0;
}









